import java.util.*;

/*
* This file is part of the XKCD Velociraptor Simulation originally written by Matthew Beckler, Grant Miller, and John Saxton.
* 
* the XKCD Velociraptor Simulation is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* The XKCD Velociraptor Simulation is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with the XKCD Velociraptor Simulation; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**/

public class Raptor extends Movable
{
	private final double ACCELERATION = 4.0; // max acceleration as defined by the problem
	
	public double currentVelocity;
	private double originalX;
	private double originalY;
	public Victim target = null;
	private double minDx, minDy;
	private Random rand = new Random();
	private double initVelocity = 0;
	public Raptor(double v, double x, double y)
	{
		super(v, x, y);
		originalX = x;
		originalY = y;
		currentVelocity = 0;
		//v = v/2;
		minDx = rand.nextInt(15) + 1;
		minDy = rand.nextInt(15) + 1;		
	}
	public Raptor(double v, double x, double y, double initVelocity)
	{
		super(v, x, y);
		originalX = x;
		originalY = y;
		currentVelocity = 0;
		//v = v/2;
		minDx = rand.nextInt(15) + 1;
		minDy = rand.nextInt(15) + 1;
		this.initVelocity = initVelocity;
	}
	
	public void reset() //used at the end of each run, to save us from having to rebuild the raptors
	{
		x = originalX;
		y = originalY;
		currentVelocity = initVelocity;
	}
	
	public boolean updatePosition(double dt, ArrayList<Victim> targets)
	{
		//After consulting the XKCD author, we have decided to remember only a linear velocity, and allow very speedy turns for the velociraptors. Perhaps this isn't a very good approximation, but for now, we'll allow the raptors to accellerate at the given rate until they reach max velocity.
		double dx, dy, distance, minDistance;
		dx = 0;
		dy = 0;
		distance = 0;
		minDistance = 3000;
		if (target == null)
		{
			for (int i = 0; i < targets.size(); i++)
			{
			   dx = targets.get(i).getX() - this.getX();
				dy = targets.get(i).getY() - this.getY();
				distance = Math.sqrt(dx * dx + dy * dy);
				if (distance < minDistance && distance <= 40)
				{
					minDx = dx;
					minDy = dy;
					minDistance = distance;
					target = (Victim)targets.get(i);
				}	
			}
		}
		else
		{
			minDx = target.getX() - this.getX();
			minDy = target.getY() - this.getY();
			minDistance = Math.sqrt(minDx * minDx + minDy * minDy);
			if (minDistance > 40)
			{
				target = null;
				//v = v/2;
			}
		}
		
		//System.out.println("updatePosition: distance = " + distance + ", angle = " + angleToVictim);
		if (!targets.contains(target))
		{
				target = null;
				//v = v/2;
		}		
		if (minDistance < 2 && target != null) // as a sort of trade-off for allowing high-speed turns, we decided the raptors would need to be very close
		{
			//eaten :- (

			//System.out.println("  Eaten! " + target);
			minDx = rand.nextInt(20) - 10;
			minDy = rand.nextInt(20) - 10;
			//v = v/2;
			return true;
			
		}
		
		//update v
		//System.out.println("  Old velocity: " + currentVelocity);
		currentVelocity = Math.min(currentVelocity + (ACCELERATION * dt), v);
		//System.out.println("  New velocity: " + currentVelocity);

		//since the raptors can turn on a dime, even at 35 m/s, and we have already calculated the new currentVelocity, we only need to find the components of the movement
		minDistance = Math.sqrt(minDx * minDx + minDy * minDy);
		
		x += ((minDx/minDistance) * currentVelocity) * dt;
		y += ((minDy/minDistance) * currentVelocity) * dt;
		
		if (x <  ((minDx/minDistance) * currentVelocity) * dt)
		{
			minDx = rand.nextInt(15);
			minDy = 5 - rand.nextInt(10); 
		}
		if (x > 300 -  ((minDx/minDistance) * currentVelocity) * dt)
		{
			minDx = -1*rand.nextInt(15);
			minDy = 5 - rand.nextInt(10); 
		}
		if (y < ((dy/distance) * currentVelocity) * dt)
		{
			minDy = rand.nextInt(15);
			minDx = 5 - rand.nextInt(10);
		}
		if (y > 300 - ((dy/distance) * currentVelocity) * dt)
		{
			minDy = -1*rand.nextInt(15);
			minDx = 5 - rand.nextInt(10);
		}
		return false;
	}
	
	public boolean updatePosition(double dt, Movable target)
	{
		//After consulting the XKCD author, we have decided to remember only a linear velocity, and allow very speedy turns for the velociraptors. Perhaps this isn't a very good approximation, but for now, we'll allow the raptors to accellerate at the given rate until they reach max velocity.
		double dx = target.getX() - this.getX();
		double dy = target.getY() - this.getY();
		double distance = Math.sqrt(dx * dx + dy * dy);

		//System.out.println("updatePosition: distance = " + distance + ", angle = " + angleToVictim);
		
		if (distance < 0.1) // as a sort of trade-off for allowing high-speed turns, we decided the raptors would need to be very close
		{
			//eaten :- (
			//System.out.println("  Eaten!");
			return true;
		}
		
		//update v
		//System.out.println("  Old velocity: " + currentVelocity);
		currentVelocity = Math.min(currentVelocity + (ACCELERATION * dt), v);
		//System.out.println("  New velocity: " + currentVelocity);

		//since the raptors can turn on a dime, even at 35 m/s, and we have already calculated the new currentVelocity, we only need to find the components of the movement

		x += ((dx/distance) * currentVelocity) * dt;
		y += ((dy/distance) * currentVelocity) * dt;
		return false;
	}
	
	public String toString()
	{
		return "Raptor Position = (" + x + ", " + y + "), Velocity = (" + currentVelocity + ") minDx" + minDx + "minDy" + minDy;
	}
}
